from django.conf import settings


class Registry(object):
    
    def __init__(self):
        self._partitions = {}
    
    def register(self, key, app_model, expression):
        if not isinstance(app_model, basestring):
            app_model = "%s.%s" % (
                app_model._meta.app_label,
                app_model._meta.object_name
            )
        
        if key in self._partitions and app_model in self._partitions[key]:
            raise Exception("'%s' is already registered." % key)
        if app_model.split(".")[0] not in settings.INSTALLED_APPS:
            raise Exception("'%s' is not in INSTALLED_APPS" % app_model.split(".")[0])
        
        if key in self._partitions:
            self._partitions[key].update({app_model: expression})
        else:
            self._partitions[key] = {app_model: expression}
    
    def expression_for(self, key, app_model):
        return self._partitions.get(key, {}).get(app_model)


registry = Registry()


def register(key, app_model, expression):
    registry.register(key, app_model, expression)



from django import template

from partitions.utils import chop


register = template.Library()


class PartitionNode(template.Node):
    
    @classmethod
    def handle_token(cls, parser, token):
        bits = token.split_contents()
        if len(bits) != 4:
            raise template.TemplateSyntaxError("Invalid number of arguments.")
        if bits[2] != "using":
            raise template.TemplateSyntaxError("Invalid arguments.")
        return cls(
            queryset = bits[1],
            key = bits[3]
        )
    
    def __init__(self, queryset, key):
        self.queryset = template.Variable(queryset)
        self.key = template.Variable(key)
    
    def render(self, context):
        var_name = self.queryset.var
        queryset = self.queryset.resolve(context)
        key = self.key.resolve(context)
        context[var_name] = chop(queryset, key)
        return ""


@register.tag
def partition(parser, token):
    """
    Usage::
        {% partition queryset using request.get_host %}
    
    Filters the queryset by using rules defined in
    partitions.registry and indexed by the key provided.
    """
    return PartitionNode.handle_token(parser, token)



from partitions.registry import registry


def chop(queryset, by):
    app_model = "%s.%s" % (
        queryset.model._meta.app_label,
        queryset.model._meta.object_name
    )
    expression = registry.expression_for(by, app_model)
    return queryset.filter(expression) if expression is not None else queryset
